/*
 * holyshout.c
 *
 * holyshout (Broadcasting utility for icecast or shoutcast)
 *
 * Author: JunSeon Oh <hollywar@mail.holywar.net>
 * Date  : $Date: 2002/11/21 03:27:45 $ 
 *
 * $Revision: 1.3 $
 *
 * Copyright(C) 2000 JunSeon Oh.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include "holyshout.h"

// global variables //

lame_global_flags gf ;
int nal=0;
struct timeval lastint, shoutstart;

char **play_list ;
char **pointer_list ;
int play_list_num=0 ;
char listfile[0xff] ;
int shuffle=0, encode=0;

int main(int argc , char **argv)
{
   shout_conn_t conn;

   int playlist = 0 , ch ;
   int loop=0 , check=0;
   struct hostent *host ;

   signal (SIGINT, s1gnal);
   signal (SIGHUP, s1gnal);

   lame_init(&gf) ;

   shout_init_connection(&conn);
   conn.name="AP_HOLYSHOUT" ;
   conn.mount="default" ;
   conn.description="HOLYSHOUT";
   conn.url="http://www.icecast.org";

   while ((ch = getopt(argc, argv, "P:p:h:l:f:m:b:g:d:n:u:s:izr")) != -1) {
        switch (ch) {
             case 'P':
   		conn.port = atoi(optarg) ;
		check++ ;
 	        break ;
             case 'p':
   		conn.password = optarg ;
		check++ ;
 	        break ;
             case 'l':
	        playlist=2 ;
		check++ ;
		strcpy(listfile,optarg) ;
 	        break ;
             case 'z':
	        shuffle=1 ;
		break ;
             case 'f':
	        playlist=1 ;
		check++ ;
		strcpy(listfile,optarg) ;
 	        break ;
             case 'i':
	        conn.icy_compat=1 ;
		conn.ispublic=1;
		break ;
             case 'm':
	        conn.mount=optarg;
		break ;
             case 'g':
	        conn.genre=optarg;
		break ;
             case 'd':
	        conn.description=optarg;
		break ;
             case 'u':
	        conn.url=optarg;
		break ;
             case 'n':
	        conn.name=optarg;
		break ;
             case 'b':
		check++ ;
		if (!strcmp(optarg,"auto") || !strcmp(optarg,"AUTO")) {
		    encode=0 ;
		} else {
	            conn.bitrate=atoi(optarg);
                    gf.brate=conn.bitrate ;
		    encode=1; 
		}
		break ;
             case 'r':
	        loop=1 ;
		break ;
             case 's':
	        gf.mode=atoi(optarg);
		break ;
             case '?':
             default:
                usage();
 	        break ;
         }
    }

    if (argc < 8 ) usage() ;
    if (check != 4) usage() ;

    argc -= optind;
    argv += optind;

    if (argc < 1 ) usage() ;

    if (isdigit(argv[argc-1][0])) {
        conn.ip = argv[argc-1] ;
    } else {
        if (argv[argc-1] == NULL) {
	    puts("You must specify server name");
	    exit(0);
	} else if (!strcmp(argv[argc-1],"")) {
	    puts("You must specify server name");
	    exit(0);
	} else if (strstr(argv[argc-1],"http://")) {
            host = gethostbyname(&argv[argc-1][7])  ;
        } else {
            host = gethostbyname(argv[argc-1])  ;
	}
	if (host==NULL) {
	   printf("Can't resolv %s\n",argv[argc-1]) ;
	   exit(0);
        }

        conn.ip = (char *)inet_ntoa(*((struct in_addr *)host->h_addr)) ;
	if (conn.ip == NULL) {
	   printf("Can't resolv %s\n",argv[argc-1]) ;
	   exit(0);
        }
    }

   if (shout_connect(&conn)) {
       printf("Connected to server (%s) ...\n",conn.ip);
   } else {
       printf("Couldn't connect %s:%d...%i\n",conn.ip, conn.port ,conn.error);
       exit(0);
   }

   while(1) {
       if (playlist==2) {
           int j = 0 ;
           init_playlist(PL_INIT) ;
           if (shuffle==1) init_playlist(PL_SHUFFLE) ;
           for (j=0 ; j<play_list_num ; j++) {
	       if (encode==1) {
                    send_setram (conn,pointer_list[j]) ;
	       } else if (encode==0) {
                    send_setram_direct (conn,pointer_list[j]) ;
	       }
           }
	   init_playlist(PL_FREE) ;
        } else if (playlist==1) {
	   if (encode==1) {
               send_setram (conn,listfile) ;
	   } else if (encode==0) {
               send_setram_direct (conn,listfile) ;
	   }
        } else {
           usage() ;
        }
	if (loop==0) {
	    break ;
	} else {
	    puts("Loopping....");
	}
    }
    shout_disconnect(&conn);

    return 0 ;
}


int send_setram (shout_conn_t conn , char *filename) 
{
   char buff[LAME_MAXMP3BUFFER*2] ;
   int ret = 0 ;
   int buf_loc=0 ;
   int filesize= 0 ;
   register int imp3 ;
   int iread, i, check=0;
   int buff_size=MIN_BUF ;
   short int Buffer[2][1152];
   char mp3buffer[LAME_MAXMP3BUFFER];
   FILE *fp ;

   if (fp = fopen(filename,"r")) {
       fclose(fp) ;
       printf("Playing : %s\n",filename) ;
   } else {
       printf("Skipping (file does not readable): %s\n",filename) ;
       return 1 ;
   }

   gf.inPath=filename ;
   gf.outPath="Streamming server";

   gf.input_format=sf_mp3 ;
   //gf.mode_fixed=1 ;
   gf.quality=5 ;
   gf.num_channels=2;

   lame_init_infile(&gf) ;
   lame_init_params(&gf) ;
   lame_print_config(&gf) ;

   do {
	 iread = lame_readframe(&gf,Buffer) ;

	 imp3=lame_encode_buffer(&gf,Buffer[0],Buffer[1],iread, mp3buffer,(int)sizeof(mp3buffer));

	 // get the SIGINT . skipped to the next song. //
	 if (nal==1) {
	     nal=0 ;
	     break ;
	 }

	 if (imp3==-1) {
	     fprintf(stderr,"ENCODE_ERROR: mp3 buffer is not big enough... \n");
	     break ;
	 }

	 if (buf_loc >= buff_size) {
             ret = shout_send_data(&conn, buff, buf_loc);
             if (!ret) {
                 fprintf(stderr,"SEND_ERROR : %i...\n", conn.error);
                 break;
             } 
             shout_sleep(&conn);
	     buf_loc=0 ;
	     if (buff_size < MAX_BUF) buff_size += INC_BUF ;
	 } 
	 if (imp3 > 0) {
	     bcopy(mp3buffer,&buff[buf_loc],imp3) ;
	     buf_loc += imp3 ;
	 }
   } while(iread) ;

   if (buf_loc > 0) {
       ret = shout_send_data(&conn, buff, buf_loc);
       shout_sleep(&conn);
   }

   imp3=lame_encode_finish(&gf,mp3buffer,(int)sizeof(mp3buffer));
   //ret = shout_send_data(&conn, mp3buffer, imp3);
   lame_close_infile(&gf) ;
   lame_mp3_tags(&gf) ;

   puts("") ;

   return 0 ;

}

void usage(void)
{
    puts(VERSION);
    puts("");
    puts("Useage: holyshout [options] hostname\n");
    puts("* required options ( you must choose one -l or -f ) *");
    puts("	-P <port>	 - Connect to admin port on server.");
    puts("	-p <password>	 - Use specified password.");
    puts("	-l <playlist>	 - Use specified file as a playlist.");
    puts("	-f <filename>	 - Use specified file.");
    puts("	-b <bitrate>	 - Start using specified bitrate");
    puts("");
    puts("if you set the bitrate to \"auto\" , holyshout does not encoding on the fly.");
    puts("");
    puts("* optional options *");
    puts("	-g <genre>	 - Use specified genre ");
    puts("	-n <name>	 - Use specified name ");
    puts("	-d <description> - Use specified description ");
    puts("	-u <url>	 - Use specified url ");
    puts("	-m <mountpoint>	 - Use specified mount point");
    puts("	-s <mode>	 - 0 = stero,1 = jstero ,2 = dual,3 = mono");
    puts("	-r		 - go on forever (loop)");
    puts("	-i 		 - Use old icy headers (for shoutcast)");
    puts("	-z 		 - Shuffle play") ;
    puts("");
    puts("NOTE : shoutcase admin port using listiening port plus one.");
    puts("       if you are set a port to 8000 , admin port is a 8001");
    puts("");

    exit(0);
}


void s1gnal(const int sig)
{

    switch(sig) {
        case SIGINT: {
            struct timeval tv;
            gettimeofday (&tv, NULL);
            if (((tv.tv_sec + tv.tv_usec / 1000000.0) - 0.4) < (lastint.tv_sec + lastint.tv_usec / 1000000.0)) { exit(0); }
            gettimeofday (&lastint, NULL);
            fflush (stdout);
	    //fprintf(stderr,"Caught signal SIGINT. skipped to next song! \n");
	    puts("");
            fflush (stdout);
            nal=1 ;
	}
	break ;
	case SIGHUP : {
	    init_playlist(PL_FREE) ;
	    init_playlist(PL_INIT) ;
	    //fprintf(stderr,"Caught signal SIGHUP. Rehashing playlist! \n");
	    if (shuffle==1) init_playlist(PL_SHUFFLE) ;
	
	}
    }
}

int init_playlist(int type) 
{
    if (type==PL_INIT) {
        FILE *fp ;
	int i=0;
	char buf[0xff] ;

        // get the playlist count //
        if (!(fp = fopen(listfile,"r"))) {
            printf("Can't open file %s\n",listfile) ;
            exit(0);
        }
	play_list_num=0 ;
        while( (fgets(buf,0xff,fp))) {
	    if (buf[0]==NULL || buf[0]=='\n' || buf[0]=='\r') continue ;
	    play_list_num++ ;
	}
	fclose(fp) ;
	    
	// memory allocation play list //
        if (!((char **)play_list = (char **) malloc (sizeof(char*)*play_list_num))){
	    fprintf(stderr,"malloc failed") ;
	    exit(0);
	}
	// memory allocation pointer //
        if (!((char **)pointer_list = (char **) malloc (sizeof(char*)*play_list_num))){
	    fprintf(stderr,"malloc failed") ;
	    exit(0);
	}

	// get the list //
        if (!(fp = fopen(listfile,"r"))) {
            printf("Can't open file %s\n",listfile) ;
            exit(0);
        }

	i=0;
        while( (fgets(buf,0xff,fp))) {
	    if (buf[0]==NULL || buf[0]=='\n' || buf[0]=='\r') continue ;
            buf[strlen(buf)-1] = '\0' ; 
	    (char *)play_list[i] = (char *) malloc(sizeof(char)*strlen(buf)+1) ;
	    strcpy(play_list[i],buf) ;
	    pointer_list[i] = play_list[i] ;
	    i++ ;
        }
        fclose(fp) ;
        fprintf(stderr,"%d songs init done.\n",play_list_num) ;
	return 0 ;
    } else if (type==PL_SHUFFLE) {
        struct timeval tfs;
	int rand1 , rand2 , j ;
	char *tmp_pointer ;

	gettimeofday(&tfs, (struct timezone *) 0) ;
	srand(tfs.tv_usec) ;
        for (j=0 ; j<=play_list_num*2 ; j++) {
            rand1 = rand()%play_list_num ;
            rand2 = rand()%play_list_num ;

	    if (rand1 != rand2) {
	        tmp_pointer = pointer_list[rand1] ;
	        pointer_list[rand1] = pointer_list[rand2] ;
	        pointer_list[rand2] = tmp_pointer ;
	    } else {
		j--;
	    }
        }
        fprintf(stderr,"Shuffling done.\n") ;

	return 0 ;
    } else if (type==PL_FREE) {
        int i ;

	for (i=0 ;i<play_list_num ;i++) {
	    free((char *)play_list[i]) ;
	}
	free((char **)play_list) ;
	free((char **)pointer_list) ;
	return 0 ;
    }
}

int send_setram_direct (shout_conn_t conn , char *filename) 
{
   char buff[4096] ;
   int read, total , ret, file_size;
   FILE *fp ;

   if (fp = fopen(filename,"r")) {
       printf("Playing : %s\n",filename) ;
   } else {
       printf("Skipping (file does not readable): %s\n",filename) ;
       return 1 ;
   }

   fseek(fp,1,SEEK_END) ;
   file_size = ftell(fp) ;
   rewind(fp) ;

   total = 0 ;

   while(1) {
       read = fread(buff, 1 , 4096 , fp) ;
       total = total + read ;

       if (nal==1) {
	   nal=0 ;
	   break ;
       }

       if (read>0) {
           ret = shout_send_data(&conn,buff,read) ;
	   if (!ret) {
	       fprintf(stderr,"SEND_ERROR : %i...\n",conn.error) ;
	       break ;
	   } 
	   shout_sleep(&conn) ;
       } else {
           break ;
       }

       fprintf(stderr,"\rSend: %-.2f %%",((float) total / (float)file_size)*100) ;
   }


   puts("") ;

   return 0 ;

}


/*
 | $Id: holyshout.c,v 1.3 2002/11/21 03:27:45 hollywar Exp $
 |
 | Local Variables:
 | mode: c
 | mode: font-lock
 | version-control: t
 | delete-old-versions: t
 | End:
 |
 | -*- End-Of-File -*-
 */
